import logging
from threading import Lock

from airzone.utils import *
from pymodbus.client.sync import ModbusTcpClient as ModbusClient  # type: ignore


def state_value(state, address, init=0, end=15):
    """
    init and end are included on the desired slice
    """
    if state is None:
        return 0
    binary = format(state[address], '016b')
    r_init = len(binary)-1-init
    r_end = len(binary)-1-end
    kBitSubStr = binary[r_end: r_init+1]
    return int(kBitSubStr, 2)


def bit_value(state, address, bit):
    if state is None:
        return 0
    # return state_value(state, address, bit, bit)
    temp = format(state[address], '016b')
    return int(temp[len(temp)-1-bit])


def change_range_bit_value(state, address, init_bit, num_bit, value):
    if num_bit <= 0:
        return state[address]
    temp2 = list(format(state[address], '016b'))
    idx = len(temp2)-init_bit-num_bit
    idx2 = len(temp2)-init_bit
    temp = list(format(value, '0'+str(num_bit) + 'b'))
    temp2[idx:idx2] = temp
    return int("".join(temp2), 2)


def change_bit_value(state, address, bit, value):
    if value:
        ch = '1'
    else:
        ch = '0'
    temp = list(format(state[address], '016b'))
    idx = len(temp)-1-bit
    temp[idx] = ch
    return int("".join(temp), 2)


def date_as_number(date):
    l = bitfield(date.weekday() + 1)
    l = pad_left_list(l, 3, 0)
    h = bitfield(date.hour)
    h = pad_left_list(h, 5, 0)
    m = bitfield(date.minute)
    m = pad_left_list(m, 6, 0)
    value = shifting(m + h + l)
    return value


# --------------------------------------------------------------------------- #
# configure the client logging
# --------------------------------------------------------------------------- #
# FORMAT = ('%(asctime)-15s %(threadName)-15s '
#           '%(levelname)-8s %(module)-15s:%(lineno)-8s %(message)s')
# logging.basicConfig(format=FORMAT)
# log = logging.getLogger()
# log.setLevel(logging.DEBUG)
UNIT = 0x1


class Gateway():

    def __init__(self, url, port):
        """                
        Arguments:
            url {String} -- Address where the master is listening
            port {String} -- Serial port string as it is used in pyserial

        Keyword Arguments:
            discover {bool} -- True if the gateway should try to discover 
                               the devices on init (default: {True})
        """
        self._url = url
        self._port = port
        self._lock = Lock()
        self.config_client()

    def config_client(self):
        """
        Configure the serial port
        """
        with self._lock:
            self.client = ModbusClient(self._url, port=self._port)
            self.client.connect()

    # innobus doc type 3
    def read_holding_registers(self, machineid, address, num_registers):
        logging.debug(
            f'read holding registers machineId: {str(machineid)} address: {str(address)} num_registers: {str(num_registers)}')

        try:
            with self._lock:
                response = self.client.read_holding_registers(
                    address, num_registers, unit=machineid)

                logging.debug('response: ' + str(response.registers))
                return response.registers
        except:
            logging.exception('Error reading holding registers')
        return None

    def read_input_registers(self, machineid, address, num_registers):  # innobus doc type 4
        logging.debug('reading input registers: machineId:' + str(machineid) +
                      ' address: ' + str(address) + ' num_registers: ' + str(num_registers))
        try:
            with self._lock:
                response = self.client.read_input_registers(
                    address, num_registers, unit=machineid)
                logging.debug('response: ' + str(response.registers))
                return response.registers
        except:
            logging.exception('Error reading input registers')
        return None

    def write_single_register(self, machineid, address, value):
        with self._lock:
            test = self.client.write_register(address, value, unit=machineid)
            print(test)
    
    def __str__(self):
        return f'Modbus_{self._url}:{self._port}'



""" Airzone Local api integration
"""
import logging
from enum import IntEnum

import requests  # type: ignore

_LOGGER = logging.getLogger(__name__)

class OperationMode(IntEnum):
    STOP = 1
    COOLING = 2
    HEATING = 3
    FAN = 4
    DRY = 5
    AUTO = 7

    @classmethod
    def _missing_(cls, value):
        return OperationMode.STOP

class TempUnits(IntEnum):
    CELSIUS = 0
    FAHRENHEIT = 1


class Speed(IntEnum):
    AUTO = 0
    SPEED_1 = 1
    SPEED_2 = 2
    SPEED_3 = 3
    SPEED_4 = 4
    SPEED_5 = 5
    SPEED_6 = 6
    SPEED_7 = 7

    @classmethod
    def _missing_(cls, value):
        return Speed.AUTO

class Machine():

    def __init__(self, machine_ipaddr, port=3000, system_id=1, vaf_cbs=False):
        self._API_ENDPOINT = f"http://{machine_ipaddr}:{str(port)}/api/v1/hvac"
        self._machine_id = system_id
        self._machine_ip = machine_ipaddr
        self._data = {'SystemID': self._machine_id, 'ZoneID': 0}
        self._error_log = []
        self._machine_state = None
        self._zones = {}                
        self._retrieve_system_data()                
    
    @property
    def machine_state(self):
        return self._machine_state

    @machine_state.setter
    def machine_state(self, value):
        self._machine_state = value
        self.update_zones()

    def discover_zones(self):                
        self._zones = {z['zoneID']: Zone(self, z['zoneID'], z) for z in self.machine_state if z['zoneID'] != 0}

    def update_zones(self):
        if self._zones == {}:
            self.discover_zones()
        for z in self.machine_state:
            self._zones[z['zoneID']].zone_state = z

    @property
    def zones(self):
        return self._zones.values()
    
    def _handle_response(self, response):
        if response.status_code == 200:
            response_json = response.json()
            self.machine_state = response_json['data']
        elif response.status_code >= 500:
            print(f'[!] [{response.status_code}] Server Error')
            return None
        response.raise_for_status()
        
    def _retrieve_system_data(self):
        try:
            response = requests.post(url=self._API_ENDPOINT,
                                           json=self._data)
            self.handle_response(response)
        except requests.exceptions.RequestException as e:
            _LOGGER.exception(str(e))            

    def set_zone_parameter_value(self, zone_id, parameter, value):
        try:

            self._data['ZoneID'] = zone_id
            self._data[parameter] = value
            response = requests.put(url=self._API_ENDPOINT,
                                    json=self._data)
            self.handle_response(response)            
        except requests.exceptions.RequestException as e:
            _LOGGER.exception(str(e))                        
    
    def _get_zone_property(self, zone_id, prop):
        z_id = zone_id
        if z_id == 0:
            z_id = next(iter(self._zones))            
        if prop in self._zones[z_id].zone_state:
            return self._zones[z_id].zone_state[prop]        
        return None


    @property
    def speed(self):
        speed = self._get_zone_property(0, 'speed')
        if speed != None:
            return Speed(speed)
        return Speed.AUTO
    
    @speed.setter
    def speed(self, speed):
        self.set_zone_parameter_value(0, 'speed', speed)
    
    @property
    def operation_mode(self):       
        return OperationMode(self._get_zone_property(0, 'mode'))

    @operation_mode.setter
    def operation_mode(self, mode):
        self.set_zone_parameter_value(0, 'mode', mode)
    
    @property    
    def units(self):
        return TempUnits(self._get_zone_property(0, 'units'))

    @property
    def unique_id(self):
        return f'Local_Api{self._machine_id}_{str(self._machine_ip)}'
  
    def __str__(self):
        zs = "\n".join([str(z) for z in self.zones])
        return "Machine with id: " + str(self._machine_id) + \
                "Mode: " + str(self.operation_mode) + \
                "\nZones\n" + zs


class Zone:
    def __init__(self, machine, zone_id, data = {}):
        self._machine = machine
        self._zone_id = zone_id        
        self.zone_state = data
              
    def _set_parameter_value(self, prop, value):
        self._machine.set_zone_parameter_value(self._zone_id, prop, value)

    @property
    def zone_state(self):
        return self._zone_state
    
    @zone_state.setter
    def zone_state(self, value):
        self._zone_state = value
    
    def is_on(self):
        return self.zone_state['on']

    def turn_on(self):
        self._set_parameter_value('on', 1)

    def turn_off(self):
        self._set_parameter_value('on', 0)

    @property
    def signal_temperature_value(self):
        return self.zone_state['setpoint']
    
    @signal_temperature_value.setter
    def signal_temperature_value(self, setpoint):
        self._set_parameter_value('setpoint', setpoint)

    @property
    def name(self):
        return self.zone_state['name']
    
    @name.setter
    def name(self, name):
        self._set_parameter_value('name', name)

    @property
    def max_temp(self):
        return self.zone_state['maxTemp']

    @property
    def min_temp(self):
        return self.zone_state['minTemp']

    @property
    def local_temperature(self):
        return self.zone_state['roomTemp']

    @property
    def dif_current_temp(self):
        return self.signal_temperature_value - self.local_temperature

    @property
    def room_humidity(self):
        return self.zone_state['humidity']

    @property
    def air_demand(self):
        return self.zone_state['air_demand']

    @property
    def floor_demand(self):
        return self.zone_state['floor_demand']

    
    @property
    def units(self):
        return TempUnits(self.zone_state['units'])


    def __str__(self):
        return "Zone with id: " + str(self._zone_id) + \
               " Name: " + str(self.name) + \
               " Zone is On: " + str(self.is_on()) + \
               " Air demand is: " + str(self.air_demand) + \
               " Room Temp: " + str(round(self.local_temperature, 1)) + \
               " " + str(self.units) +\
               " Room humidity: " + str(self.room_humidity) + \
               " Max_Temp: " + str(self.max_temp) + \
               " Min_Temp: " + str(self.min_temp)



"""Local Api tests."""
import json
import os

import pytest  # type: ignore
import requests_mock  # type: ignore

from airzone.localapi import Machine, OperationMode, Speed

THIS_DIR = os.path.dirname(os.path.abspath(__file__))
response_test_path = os.path.join(THIS_DIR, "data/response.json")


@pytest.fixture
def mock_machine():
    """Fixture localapi Machine init with the data/response.json file."""
    with requests_mock.Mocker() as mock_resp:
        f = open(response_test_path,)
        data = json.load(f)
        machine_ipaddr = "0.0.0.0"
        mock_addr = f"http://{machine_ipaddr}:3000/api/v1/hvac"
        mock_resp.post(mock_addr, json=data)
        return Machine(machine_ipaddr)


def test_create_machine(mock_machine):
    """Test the creation of a machine and zones with a valid json."""
    assert mock_machine.speed == Speed.AUTO
    assert mock_machine.operation_mode == OperationMode.COOLING



#!/usr/bin/python
from airzone.protocol import Gateway


def airzone_factory(address, port, machineId, system="innobus", **kwargs):
    if system == 'localapi':
        from airzone.localapi import Machine
        m = Machine(address, port, machineId)
    else:
        gat = Gateway(address, port)
        if system == 'innobus':
            from airzone.innobus import Machine
            m = Machine(gat, machineId)
        else:
            from airzone.aido import Aido
            m = Aido(gat, machineId, **kwargs)    
    return m


if __name__ == '__main__':
    aido_args = {"has_louvres": False, "speed_as_per": True}
    m = airzone_factory('modbus.local', 5020, 1, "aido", **aido_args)
    # m = airzone_factory('modbus.local', 5020, 1, "aido")

    z = m.get_zones()[0]
    print(m._machine_state)
    print(z._zone_state)
    print(bool(z.is_tacto_on()))
    print(z.get_signal_temperature_value())    
    print(z.is_floor_active())
    print(z.is_sleep_on())
    print(z.is_automatic_mode())
    print(z.get_zone_mode())
    from airzone.protocol import state_value

    print(state_value(z._zone_state, 0, 0, 1))
    print(format(z._zone_state[0], '016b'))

    ## Localapi
    # Lines for Tests. Adapt argument ip address and system id (1 == standard).
    #m = airzone_factory('192.168.90.9', 3000, 1, "localapi")  
    #print("Printing Post JSON data")
    #print(m.machine_state)
    #print("Number of zones: ", len(m.zones))
    #print(m)



import base64

def bitfield(n):
    '''
    Obtains the binary array from the number
    '''
    return [1 if digit=='1' else 0 for digit in bin(n)[2:]]

def shifting(bitlist):
    '''
    Obtain the number from the binary array
    '''
    out = 0
    for bit in bitlist:
        out = (out << 1) | bit
    return out

def true_in_list(l):
    return [i for i,v in enumerate(l) if v]

def pad_left_list(l, size, pad_value):
    for n in range(len(l), size):
        l = [pad_value] + l
    return l

def pad_right_list(l, size, pad_value):
    for n in range(len(l), size):
        l = l + [pad_value]
    return l

# toggleBit() returns an integer with the bit at 'offset' inverted, 0 -> 1 and 1 -> 0.
def toggleBit(int_type, offset):
    mask = 1 << offset
    return(int_type ^ mask)

# setBit() returns an integer with the bit at 'offset' set to 1.

def setBit(int_type, offset):
    mask = 1 << offset
    return(int_type | mask)

# clearBit() returns an integer with the bit at 'offset' cleared.

def clearBit(int_type, offset):
    mask = ~(1 << offset)
    return(int_type & mask)





import datetime
from enum import Enum, IntEnum

from airzone.protocol import *
from deprecated import deprecated  # type: ignore


class OperationMode(Enum):
    STOP = 0
    COLD = 1
    HOT = 2
    AIR = 3
    HOT_AIR = 4
    HOTPLUS = 258


class ZoneMode(Enum):
    MANUAL = 0
    MANUAL_SLEEP = 1
    AUTOMATIC = 2
    AUTOMATIC_SLEEP = 3


class FancoilSpeed(IntEnum):
    AUTOMATIC = 0
    SPEED_1 = 1
    SPEED_2 = 2
    SPEED_3 = 3
    
    @classmethod
    def _missing_(cls, value):
        return FancoilSpeed.AUTOMATIC


class ProtectionTime(Enum):
    TEN_SEC = 0
    FOUR_MIN = 1


class VentilationMode(Enum):
    CONTINUOUS = 0
    AUTOMATIC = 1


class LastGridCloseMode(Enum):
    TIMER = 0
    NO_TIMER = 1


class GridMode(Enum):
    All_NONE = 0
    PROPORTIONAL = 1


class GridAngle(Enum):
    NINETY = 0
    FIFTY = 1
    FORTY_FIVE = 2
    FORTY = 3


class ProbeType(Enum):
    NO_PROBE = 0
    REMOTE = 1
    FLOOR = 2


class RelayConfig(Enum):
    OFF = 0
    USUALLY_OPEN = 1
    USUALLY_CLOSE = 2


class LocalFancoilType(Enum):
    GRID = 0
    FANCOIL = 1


class Machine():

    def __init__(self, gateway, machineId):
        self._gateway = gateway
        self._machineId = machineId
        self._machine_state = None
        self.sync_clock(True)
        self._zones = {}        
        self._retrieve_machine_state()

        
    @property
    def machine_state(self):
        return self._machine_state

    @machine_state.setter
    def machine_state(self, value):
        self._machine_state = value
        self.update_zones()

    def discover_zones(self):
        zones = self.read_registers(9, 2)
        config_zones1 = true_in_list(list(reversed(bitfield(zones[0]))))
        config_zones2 = [
            v + 8 for v in true_in_list(list(reversed(bitfield(zones[1]))))]
        config_zones = config_zones1 + config_zones2
        self._zones = {i+1: Zone(self, i + 1) for i in config_zones}
    

    def update_zones(self):
        if self._zones == {}:
            self.discover_zones()
        for zone in self._zones.values():
            zone.retrieve_zone_status()

    @property            
    def zones(self):
        return self._zones.values()

    @deprecated('use property')
    def get_zones(self):
        return self.zones

    def read_registers(self, address, numRegisters):
        return self._gateway.read_input_registers(
            self._machineId, address, numRegisters)

    def write_register(self, address, value):
        return self._gateway.write_single_register(
            self._machineId, address, value)

    def _retrieve_machine_state(self, retrieve_zones=True):
        self.machine_state = self.read_registers(0, 21)

    def sync_clock(self, force=False):
        current_clock = self.read_registers(4, 1)
        if current_clock[0] == 0 or force:
            self.set_clock()

    def set_clock(self):
        d = datetime.datetime.now()
        value = date_as_number(d)
        self.write_register(4, value)

    @property
    def operation_mode(self):
        if self._machine_state == None:
            return OperationMode.STOP
        return OperationMode(self._machine_state[0])

    
    @operation_mode.setter
    def operation_mode(self, operationMode):
        self.write_register(0, OperationMode[OperationMode].value)
    

    @deprecated('use property')
    def get_operation_mode(self):
        return self.operation_mode


    @deprecated('use property')
    def set_operation_mode(self, operationMode):
        self.operation_mode = operationMode

    @property
    def hotplus_differential_signal(self):
        return state_value(self._machine_state, 2)

    @deprecated('use property')
    def get_hotplus_differential_signal(self):
        return self.hotplus_differential_signal

    @property
    def protection_time(self):
        return state_value(self._machine_state, 3, 0, 0)

    @deprecated('use property')
    def get_protection_time(self):
        return self.protection_time

    def central_relay_state_1(self):
        if self._machine_state == None:
            return bitfield(0)
        return bitfield(self._machine_state[13])

    def __str__(self):
        zs = "\n".join([str(z) for z in self.get_zones()])
        return "Machine with id: " + str(self._machineId) + \
               "Zones: \n" + zs

    @property
    def unique_id(self):
        return f'Innobus_M{self._machineId}_{str(self._gateway)}'


    @deprecated('Use the machine_state property instead')
    def get_machine_state(self):
        return self._machine_state



class Zone():

    def __init__(self, machine, zone_id):
        self._machine = machine
        self.zoneId = zone_id    
        self.base_zone = zone_id * 256
        self._zone_state = None
        self.retrieve_zone_state()

    def write_register(self, address, value):
        return self._machine.write_register(self.base_zone + address, value)

    def write_bit_value(self, address, bit, value):
        new_value = change_bit_value(self._zone_state, address, bit, value)
        self.write_register(address, new_value)

    def __str__(self):
        return "Zone with id: " + str(self.zone_id) + \
               " ZoneMode: " + str(self.get_zone_mode()) + \
               " Tacto On: " + str(self.is_tacto_on()) + \
               " Hold On: " + str(self.is_zone_hold())

    @property
    def zone_state(self):
        return self._zone_state
    
    @zone_state.setter
    def zone_state(self, value):
        self._zone_state = value

    def retrieve_zone_state(self):
        self.zone_state = self._machine.read_registers(self.base_zone, 13)

    # OPERATION ZONE MODE
    def is_sleep_on(self):
        return bit_value(self._zone_state, 0, 0)

    def turnon_sleep(self):
        self.write_bit_value(0, 0, 1)

    def turnoff_sleep(self):
        self.write_bit_value(0, 0, 0)

    def is_automatic_mode(self):
        return bit_value(self._zone_state, 0, 1)

    def turnon_automatic_mode(self):
        self.write_bit_value(0, 1, 1)

    def turnoff_automatic_mode(self):
        self.write_bit_value(0, 1, 0)

    def get_zone_mode(self):
        temp = state_value(self._zone_state, 0, 0, 1)
        return ZoneMode(temp)

    def set_zone_mode(self, zoneMode):
        zm = ZoneMode[zoneMode]
        if zm == ZoneMode.MANUAL:
            self.turnoff_automatic_mode()
            self.turnoff_sleep()
        elif zm == ZoneMode.MANUAL_SLEEP:
            self.turnoff_automatic_mode()
            self.turnon_sleep()
        elif zm == ZoneMode.AUTOMATIC:
            self.turnon_automatic_mode()
            self.turnoff_sleep()
        elif zm == ZoneMode.AUTOMATIC_SLEEP:
            self.turnon_automatic_mode()
            self.turnon_sleep()

        # temp = change_range_bit_value(self._zone_state, 0, 0, 2, ZoneMode[zoneMode].value)
        # self.write_register(0, temp)

    def is_tacto_on(self):
        return bit_value(self._zone_state, 0, 2)

    def turnon_tacto(self):
        self.write_bit_value(0, 2, 1)

    def turnoff_tacto(self):
        self.write_bit_value(0, 2, 0)

    def is_zone_hold(self):
        return bit_value(self._zone_state, 0, 3)

    def turnon_hold(self):
        self.write_bit_value(0, 3, 1)

    def turnoff_hold(self):
        self.write_bit_value(0, 3, 0)

    def get_speed_selection(self):
        temp = state_value(self._zone_state, 0, 4, 5)
        return FancoilSpeed(temp)

    def set_speed_selection(self, fancoilSpeed):
        temp = change_range_bit_value(self._zone_state, 0, 4, 5, FancoilSpeed[fancoilSpeed].value)
        self.write_register(0, temp)

    ####

    @property
    def min_temp(self):
        if self._zone_state == None:
            return -1
        return self._zone_state[1] / 10 
    
    @min_temp.setter
    def min_temp(self, value):
        if value >= 18 or value <= 22:
            self.write_register(1, value * 10)
    
    
    @deprecated('Use min_temp')
    def get_min_signal_value(self):
        return self.min_temp        
    
    @deprecated('Use min_temp')
    def set_min_signal_value(self, value):
        self.min_temp = value

    @property
    def max_temp(self):
        if self._zone_state == None:
            return -1
        return self._zone_state[2] / 10

    @max_temp.setter
    def max_temp(self, value):
        if value >= 25 or value <= 30:
            self.write_register(2, value * 10)      

    @deprecated('Use max_temp')
    def get_max_signal_value(self):
        return self.max_temp

 

    @deprecated('Use max_temp')
    def set_max_signal_value(self, value):
        self.max_temp = value        

    @property
    def signal_temperature_value(self):
        if self.zone_state == None:
            return -1
        return self.zone_state[3] / 10
    
    @signal_temperature_value.setter
    def signal_temperature_value(self, value):
        if value >= 18 or value <= 30:
            self.write_register(3, int(value * 10))   

    @deprecated('use property')
    def get_signal_temperature_value(self):
        return self.signal_temperature_value


    @deprecated('use property')
    def set_signal_temperature_value(self, value):
        self.signal_temperature_value = value
        

    # ZONE CONFIGURATION

    def is_master_zone(self):
        return bit_value(self._zone_state, 4, 0)

    def get_grid_mode(self):
        temp = bit_value(self._zone_state, 4, 1)
        return GridMode(temp)

    def is_AA_enabled(self):
        return bit_value(self._zone_state, 4, 2)

    def is_Floor_enabled(self):
        return bit_value(self._zone_state, 4, 3)

    def get_grid_angle_hot(self):
        temp = state_value(self._zone_state, 4, 5, 6)
        return GridAngle(temp)

    def get_grid_angle_cold(self):
        temp = state_value(self._zone_state, 4, 7, 8)
        return GridAngle(temp)

    def get_is_minimun_air_enabled(self):
        temp = bit_value(self._zone_state, 4, 9)
        return temp

    def get_probe_type(self):
        temp = state_value(self._zone_state, 4, 10, 11)
        return ProbeType(temp)

    def get_presence(self):
        temp = state_value(self._zone_state, 4, 12, 13)
        return RelayConfig(temp)

    def get_window(self):
        temp = state_value(self._zone_state, 4, 14, 15)
        return RelayConfig(temp)

    ######

    def get_grid_opened_time(self):
        return state_value(self._zone_state, 5) * 10

    def get_tacto_address(self):
        return state_value(self._zone_state, 6)

    def get_master_tacto_address(self):
        return state_value(self._zone_state, 7)

    def get_remote_probe_temperature(self):
        return self._zone_state[8] / 10

    # Zone state register
    def is_zone_grid_opened(self):
        return bit_value(self._zone_state, 9, 0)

    def is_grid_motor_active(self):
        return bit_value(self._zone_state, 9, 1)

    def is_grid_motor_requested(self):
        return bit_value(self._zone_state, 9, 2)

    def is_floor_active(self):
        return bit_value(self._zone_state, 9, 5)

    def get_local_module_fancoil(self):
        temp = bit_value(self._zone_state, 9, 6)
        return LocalFancoilType(temp)

    def is_requesting_air(self):
        return bit_value(self._zone_state, 9, 7)

    def is_occupied(self):
        return bit_value(self._zone_state, 9, 8)

    def is_window_opened(self):
        return bit_value(self._zone_state, 9, 9)

    def get_fancoil_speed(self):
        temp = state_value(self._zone_state, 9, 10, 11)
        return FancoilSpeed(temp)

    def get_proportional_aperture(self):
        return state_value(self._zone_state, 9, 12, 13)

    def is_tacto_connected_cz(self):
        return bit_value(self._zone_state, 9, 14)

    ###

    @property
    def local_temperature(self):
        return self._zone_state[10] / 10

    @deprecated('use property')
    def get_local_temperature(self):
        return self._zone_state[10] / 10

    @property
    def dif_current_temp(self):
        return self.signal_temperature_value - self.local_temperature()
    
    @deprecated('use property')
    def get_dif_current_temp(self):
        return self.dif_current_temp

    @property
    def unique_id(self):
        return f'{str(self._machine)}_Z{self.zoneId}'



from enum import IntEnum

from deprecated import deprecated  # type: ignore


class OperationMode(IntEnum):
    AUTO = 1
    COOLING = 2
    HEATING = 3
    FAN = 4
    DRY = 5

    @classmethod
    def _missing_(cls, value):
        return OperationMode.AUTO

class Speed(IntEnum):
    AUTO = 0
    SPEED_1 = 1
    SPEED_2 = 2
    SPEED_3 = 3
    SPEED_4 = 4
    SPEED_5 = 5
    SPEED_6 = 6
    SPEED_7 = 7

    @classmethod
    def _missing_(cls, value):
        return Speed.AUTO

class Louvres(IntEnum):
    POS_0 = 0
    POS_1 = 1
    POS_2 = 2
    POS_3 = 3
    POS_4 = 4
    POS_5 = 5
    POS_6 = 6
    POS_7 = 7
    AUTO = 8
    SWING = 9
    SWIRL = 10

    @classmethod
    def _missing_(cls, value):
        return Louvres.AUTO



class Aido():

    def __init__(self, gateway, machineId, has_louvres = True, speed_as_per = False):
        self._gateway = gateway
        self._machineId = machineId
        self._machine_state = None        
        self._has_louvres = has_louvres
        self._speed_as_per = speed_as_per        

        self._retrieve_machine_state()
    
    def _read_registers(self, address, numRegisters):
        return self._gateway.read_input_registers(
            self._machineId, address, numRegisters)
    
    def _write_register(self, address, value):
        return self._gateway.write_single_register(
            self._machineId, address, value)

    def _retrieve_machine_state(self):
        self._machine_state = self._read_registers(0, 7)
    
    def get_is_machine_on(self):
        if self._machine_state == None:
            return 0
        return self._machine_state[0]
    
    def turn_on(self):
        self._write_register(0, 1)
    
    def turn_off(self):
        self._write_register(0, 0)

    def get_signal_temperature_value(self):
        if self._machine_state == None:
            return -1
        return self._machine_state[1] / 10
    
    def set_signal_temperature_value(self, value):
        if value >= 18 or value <= 30:
            self._write_register(1, int(value * 10))

    def get_local_temperature(self):
        return self._machine_state[2] / 10
    
    def get_operation_mode(self):
        if self._machine_state == None:
            return OperationMode.AUTO
        return OperationMode(self._machine_state[3])

    def set_operation_mode(self, operationMode):
        if not self.get_is_machine_on():
            self.turn_on()
        self._write_register(3, OperationMode[operationMode].value)
    
    def get_speed(self):
        if self._machine_state == None:
            return Speed.AUTO
        value = self._machine_state[4]
        #TODO: review for different machines
        if self._speed_as_per:
            value = value * 4 // 100
        return Speed(value)
      
    def set_speed(self, speed):
        value = Speed[speed].value
        if self._speed_as_per:
            value = value*100 // 4

        self._write_register(4, value)
    
    def get_speed_steps(self):
        if self._speed_as_per:
            return 4
        return 7

    def has_louvres(self):
        return self._has_louvres

    def get_louvres(self):
        if self._machine_state == None:
            return Louvres.AUTO
        return Louvres(self._machine_state[5])

    def set_louvres(self, louvre):
        self._write_register(5, Louvres[louvre].value)
    
    #TODO Errors and warnings

    def __str__(self):
        
        return "Aido with id: " + str(self._machineId) + \
            "On:" +  str(self.get_is_machine_on) + \
            "Operation Mode: " + str(self.get_operation_mode()) + \
            "Signal Temp: " + str(self.get_signal_temperature_value()) + \
            "Local Temp: " + str(self.get_local_temperature()) + \
            "Speed: " + str(self.get_speed()) + \
            "Louvres: " + str(self.get_louvres())
        
    
    def unique_id(self):
        return f'Aido_M{self._machineId}_{str(self._gateway)}'

    
    @property
    def machine_state(self):
        return self._machine_state

    @deprecated('Use the machine_state property instead')    
    def get_machine_state(self):
        return self._machine_state



#!/usr/bin/env python
import argparse

from importlib_metadata import PackageNotFoundError, version  # type: ignore

import airzone

try:
    __version__ = version("python-airzone")
except PackageNotFoundError:
    # package is not installed
    pass


def action(args):
    m = airzone.airzone_factory(args.address, args.port, args.machine, args.system)
    if args.state == 'str':
        print(str(m))
    else:
        print(str(m.machine_state))


parser = argparse.ArgumentParser(prog='airzone')
parser.add_argument('-v', '--version', action='version', version='%(prog)s ' + __version__)
parser.add_argument("address", type=str, help="serial device or ip address for localapi")
parser.add_argument("port", type=str, help="serial tcp port or http port for localapi")
parser.add_argument("--machine", type=int, default=1, help="Machine number where connect")
parser.add_argument("--system", choices=['innobus', 'aido', 'localapi'], default='innobus', help="Type of Airzone System")
parser.add_argument("--state", choices=['str', 'raw'], default='str',
                    help="Get the formatted state, or the raw machine state")
parser.set_defaults(func=action)

args = parser.parse_args()
args.func(args)
